# -*- coding: utf-8 -*-
import argparse


class ArgumentParser(argparse.ArgumentParser):
    """ArgumentParser with optional logging, config, and sub-command support."""

    def __init__(self, add_log_args=False, config_opts=None, **kwargs):
        super().__init__(**kwargs)

        if add_log_args:
            from . import _logging
            _logging.addCommandLineArgs(self)
        self._add_log_args = add_log_args

        if config_opts:
            from . import _config
            _config.addCommandLineArgs(self, config_opts)
        self._config_opts = config_opts

    def parse_known_args(self, args=None, namespace=None):
        from . import _logging

        parsed, remaining = super().parse_known_args(args=args,
                                                     namespace=namespace)

        if "config" in parsed and "config_overrides" in parsed:
            config = parsed.config
            overrides = parsed.config_overrides or []
            for sect, subst in overrides:
                key, val = subst
                if not config.has_section(sect):
                    config.add_section(sect)
                config.set(sect, key, val)

        if "config_show_default" in parsed and parsed.config_show_default:
            # Not using the msg facility of self.exit because it goes to stderr
            print(self._config_opts.default_config)
            self.exit(0)
            # does not return

        if self._add_log_args:
            parsed.applyLoggingOpts = _logging.applyLoggingOpts

        return parsed, remaining

    def add_subparsers(self, add_help_subcmd=False, **kwargs):
        subparser = super().add_subparsers(**kwargs)
        if add_help_subcmd:
            # 'help' subcommand; turns it into the less intuitive --help format.
            # e.g.  cmd help subcmd  ==> cmd subcmd --help
            def _help(args, config):
                if args.command:
                    self.parse_args([args.command, "--help"])
                else:
                    self.print_help()
                self.exit(0)

            help = subparser.add_parser("help",
                                        help="Show help for a sub command")
            help.set_defaults(func=_help)
            help.add_argument("command", nargs='?', default=None)

        return subparser



import asyncio
from .app import AsyncApplication


class Application(AsyncApplication):

    def __init__(self, main_func=None, *, event_loop=None, **kwargs):
        super().__init__(main_func, **kwargs)
        self.event_loop = event_loop or asyncio.get_event_loop()
        self._main_task = None
        self._exit_status = None

    def _run(self, args_list=None):
        self.log.debug("aio.Application: {args_list}".format(**locals()))
        self._main_task = \
            self.event_loop.create_task(self.main(args_list=args_list))
        try:
            self._exit_status = \
                self.event_loop.run_until_complete(self._main_task)
        except asyncio.CancelledError as ex:
            self.log.debug("aio.Application: Cancelled: {}"
                           .format(ex))
        return self._exit_status

    def stop(self, exit_status=0):
        self.log.debug("Application::stop(exit_status=%d)" % exit_status)
        self._exit_status = exit_status
        self._main_task.cancel()



import curio
from .app import AsyncApplication


class Application(AsyncApplication):
    def __init__(self, main_func=None, *, with_monitor=False, **kwargs):
        super().__init__(main_func, **kwargs)
        self._monitor = with_monitor

    def _run(self, args_list=None):
        self.log.debug("curio.Application: {args_list}".format(**locals()))
        retval = curio.run(self.main(args_list=args_list),
                           with_monitor=self._monitor)
        return retval



# -*- coding: utf-8 -*-
import pytest
from pathlib import Path


@pytest.fixture
def unshallowed_repo():
    src = "./.git/shallow"
    bak = src + ".BACKUP"

    shallow_file = Path(src)
    if shallow_file.exists():
        shallow_file.rename(bak)
        yield shallow_file
        shallow_file = Path(bak).rename(src)
    else:
        yield None



import os
import types
import logging
from unittest.mock import patch
import pytest
from nicfit.console import pout, perr
from nicfit.console.ansi import Fg, Bg, Style, init as ansi_init
import contextlib


@contextlib.contextmanager
def envvars(*vars):
    saved = {}
    for v in vars:
        saved[v] = os.environ[v] if v in os.environ else None
    try:
        yield
    finally:
        for v in saved:
            if saved[v]:
                os.environ[v] = saved[v]
            elif v in os.environ:
                del os.environ[v]


def test_pout(capfd):
    msg = "There's a war outside!"
    pout(msg)
    out, _ = capfd.readouterr()
    assert out == msg + "\n"


def test_perr(capfd):
    msg = "There's a war outside!"
    perr(msg)
    _, err = capfd.readouterr()
    assert err == msg + "\n"


def test_plog(capfd):
    msg = "Xymox - Phoenix"

    log = logging.getLogger("test_plog")
    with patch.object(log, "info") as mock:
        pout(msg, log=log)
    out, _ = capfd.readouterr()
    assert out == msg + "\n"
    mock.assert_called_once_with(msg)


def test_ansi_unsupported():
    with envvars("TERM", "OS"):
        for var, invalids in [("TERM", ["dumb"]), ("OS", ["Windows_NT"])]:
            for val in invalids:
                os.environ[var] = val
                with pytest.raises(ValueError):
                    ansi_init(True)


def test_ansi():
    CSI = "\033["

    class KnownFg:
        (GREY,
         RED,
         GREEN,
         YELLOW,
         BLUE,
         MAGENTA,
         CYAN,
         WHITE) = [*range(30, 38)]
        RESET = 39

    class KnownBg:
        (GREY,
         RED,
         GREEN,
         YELLOW,
         BLUE,
         MAGENTA,
         CYAN,
         WHITE) = [*range(40, 48)]
        RESET = 49

    class KnownStyle:
        (RESET_ALL,
         BRIGHT,
         DIM,
         ITALICS,
         UNDERLINE,
         BLINK_SLOW,
         BLINK_FAST,
         INVERSE) = [*range(0, 8)]
        STRIKE_THRU = 9
        (RESET_BRIGHT,
         RESET_ITALICS,
         RESET_UNDERLINE,
         RESET_BLINK_SLOW,
         RESET_BLINK_FAST,
         RESET_INVERSE) = [*range(22, 28)]
        RESET_STRIKE_THRU = 29
        RESET_DIM = RESET_BRIGHT

    def mkcode(c):
        return "{}{}m".format(CSI, c)

    ansi_init(True)

    for known_palette, palette in ((KnownFg, Fg),
                                   (KnownBg, Bg),
                                   (KnownStyle, Style),
                                  ):
        code_list = [c for c in dir(known_palette) if c == c.upper()]

        # Test values and members
        if known_palette in (KnownFg, KnownBg):
            assert len(code_list) == 9
        else:
            assert len(code_list) == 17

        for c in code_list:
            assert type(getattr(palette, c)) is str
            assert mkcode(getattr(known_palette, c)) == \
                getattr(palette, c)
            if palette is Style and c.lower().startswith("reset_"):
                # Style.reset_*() functions don't exist
                continue

            assert isinstance(getattr(palette, c.lower()), types.FunctionType)

            # Test palette functions vs codes
            assert Fg.BLUE + "SNFU" + Fg.RESET == Fg.blue("SNFU")

            code = getattr(palette, c)
            if palette is Style:
                reset = getattr(palette, "RESET_{}".format(c))
            else:
                reset = getattr(palette, "RESET")
            func = getattr(palette, c.lower())

            assert code + "SNFU" + reset == func("SNFU")


def test_ansi_palette_attr_disabled():
    ansi_init(False)
    for palette in (Fg, Bg, Style):
        code_list = [c for c in dir(palette) if c == c.upper()]

        for c in code_list:
            assert getattr(palette, c) == ""


def test_ansi_formats():
    ansi_init(True)

    s = "Heavy Cream - Run Free"
    assert Fg.green(s, Style.BRIGHT,
                       Style.UNDERLINE,
                       Style.ITALICS) == \
           Style.BRIGHT + Style.UNDERLINE + Style.ITALICS + Fg.GREEN + s + \
           Fg.RESET + Style.RESET_ALL

    print("%(BLUE)sNice%(RESET)s" % Fg)

    # TODO: More complex examples and future format schems



import os
import tempfile
import argparse
import configparser
from pathlib import Path
from os.path import expandvars, expanduser, join

import pytest

from nicfit import Config
from nicfit import ConfigOpts
from nicfit import ArgumentParser
from nicfit._config import ConfigFileType, _config_override


def test_ConfigOpts():
    opts = ConfigOpts()
    assert not opts.required
    assert opts.default_file is None
    assert opts.default_config is None
    assert opts.ConfigClass is Config

    class MyConfig(Config):
        pass
    opts = ConfigOpts(required=True, ConfigClass=MyConfig,
                      default_file="config.ini", default_config="foobar")
    assert opts.required
    assert opts.default_file is "config.ini"
    assert opts.default_config is "foobar"
    assert opts.ConfigClass is MyConfig

    config = Config("/tmp/config.ini")
    assert(str(config.filename) == "/tmp/config.ini")
    config = Config("$HOME/config.ini")
    assert(str(config.filename) == join(expandvars("$HOME"), "config.ini"))
    config = Config("~/config.ini")
    assert(str(config.filename) == join(expanduser("~"), "config.ini"))
    config = Config("~/${HOME}/config.ini")

    filename = filename2 = None
    fn, filename = tempfile.mkstemp()
    os.close(fn)
    try:
        config = Config(filename)

        config.add_section("1")
        config.set("1", "one", "uno")
        config.add_section("2")
        config.set("2", "two", "dos")
        config.add_section("3")
        config.set("3", "three", "tres")

        config.write()

        config2 = Config(filename)
        config2.read()
        assert(config2.get("1", "one") == "uno")
        assert(config2.get("2", "two") == "dos")
        assert(config2.get("3", "three") == "tres")

        fn, filename2 = tempfile.mkstemp()
        os.close(fn)

        with open(filename2, 'w') as fp:
            config2.write(fileobject=fp)
        config3 = Config(filename2)
        config3.read()
        assert(config3.get("1", "one") == "uno")
        assert(config3.get("2", "two") == "dos")
        assert(config3.get("3", "three") == "tres")

    finally:
        if filename is not None:
            os.unlink(filename)
        if filename2 is not None:
            os.unlink(filename2)


def test_ConfigConstructor():
    c = Config("file.ini")
    assert isinstance(c, configparser.ConfigParser)
    assert isinstance(c.filename, Path)
    assert str(c.filename) == "file.ini"

    os.environ["var1"] = "foo"
    os.environ["var2"] = "bazz"
    c = Config("~/dir/${var1}/${var2}/config.ini")
    assert str(c.filename) == os.path.expanduser("~/dir/foo/bazz/config.ini")


def test_ConfigRead(tmpdir):
    filename = os.path.join(str(tmpdir), "config.ini")

    # File not found
    c = Config(filename)
    with pytest.raises(FileNotFoundError):
        c.read()

    # File not found, but use touch option
    assert not Path(filename).exists()
    c.read(touch=True)
    assert Path(filename).exists()
    assert c.sections() == []

    with open(filename, "w") as fp:
        fp.write(SAMPLE_CONFIG1)
    c = Config(filename)
    c.read()
    assert set(c.sections()) == {"MAIN", "CONFIG1"}


def test_ConfigReadMulti(tmpdir):
    all_files = []
    for i, cfg in enumerate([SAMPLE_CONFIG1, SAMPLE_CONFIG2,
                             SAMPLE_CONFIG3], 1):
        filename = os.path.join(str(tmpdir), "config{:d}.ini".format(i))
        with open(filename, "w") as fp:
            fp.write(cfg)
        all_files.append(filename)

    filename = os.path.join(str(tmpdir), "config4.ini")
    c = Config(filename)
    c.read(filenames=all_files, touch=True)
    assert set(c.sections()) == {"MAIN", "CONFIG1", "CONFIG2", "CONFIG3"}
    assert c.get("MAIN", "mainkey") == "config3"

    all_files.reverse()

    c = Config(filename)
    c.read(filenames=all_files, touch=True)
    assert set(c.sections()) == {"MAIN", "CONFIG1", "CONFIG2", "CONFIG3"}
    assert c.get("MAIN", "mainkey") == "config1"


def test_ConfigWrite(tmpdir):
    filename = os.path.join(str(tmpdir), "config.ini")
    c = Config(filename)
    c.read_string(SAMPLE_CONFIG1)
    c.write()

    c2 = Config(filename)
    c2.read()
    assert [i for i in c.items()] == [i for i in c2.items()]

    fpfile = os.path.join(str(tmpdir), "configfp.ini")
    with open(fpfile, "w") as fp:
        c2.write(fp)
    c3 = Config(fpfile)
    c3.read()
    assert [i for i in c3.items()] == [i for i in c2.items()]


def test_ConfigFileType(tmpdir):
    cfgtype = ConfigFileType()
    assert isinstance(cfgtype, argparse.FileType)
    assert cfgtype._encoding == "utf-8"
    assert cfgtype._opts.default_config == None

    # File does not exist
    f = os.path.join(str(tmpdir), "dne")
    with pytest.raises(argparse.ArgumentTypeError):
        cfgtype(f)

    # File does not exist, but provide a default config
    f = os.path.join(str(tmpdir), "default")
    cfgtype = ConfigFileType(ConfigOpts(default_config=SAMPLE_CONFIG3))
    assert cfgtype._opts.default_config == SAMPLE_CONFIG3
    config = cfgtype(f)
    assert isinstance(config, Config)
    assert str(config.filename) == f
    config.write()
    copy = Config(f)
    copy.read()
    assert [i for i in copy.items()] == [i for i in config.items()]


    # File exists
    f = os.path.join(str(tmpdir), "exists")
    with open(f, "w") as fp:
        fp.write(SAMPLE_CONFIG3)
    cfgtype = ConfigFileType()
    existing = cfgtype(f)
    assert [i for i in existing.items()] == [i for i in config.items()]


def test_ConfigArgumentParser(tmpdir):
    # A config is optional, so a -c/--config arg was added
    p = ArgumentParser(config_opts=ConfigOpts())
    args = p.parse_args([])

    # No arg value
    with pytest.raises(SystemExit):
        args = p.parse_args(["-c"])
    with pytest.raises(SystemExit):
        args = p.parse_args(["--config"])

    # Arg value, but config does not exist
    with pytest.raises(SystemExit):
        args = p.parse_args(["-c", "dne"])
    with pytest.raises(SystemExit):
        args = p.parse_args(["--config", "dne"])

    # Arg value, config does not exist, but a default was given.
    p = ArgumentParser(config_opts=ConfigOpts(default_config=SAMPLE_CONFIG1))
    f = os.path.join(str(tmpdir), "default.ini")
    args = p.parse_args(["-c", f])
    assert args.config is not None
    sample1 = configparser.ConfigParser()
    sample1.read_string(SAMPLE_CONFIG1)
    assert [i for i in args.config.items()] == [i for i in sample1.items()]

    # A config is required, so positional config argument is added
    p = ArgumentParser(config_opts=ConfigOpts(required=True))
    with pytest.raises(SystemExit):
        args = p.parse_args([])

    f = os.path.join(str(tmpdir), "default.ini")
    # Arg value, but config does not exist
    with pytest.raises(SystemExit):
        args = p.parse_args([f])

    sample3 = Config(f)
    sample3.read_string(SAMPLE_CONFIG3)
    sample3.write()
    args = p.parse_args([f])
    assert [i for i in args.config.items()] == [i for i in sample3.items()]


def test_ConfigOverrides(tmpdir):
    sample2 = Config("sample2")
    sample2.read_string(SAMPLE_CONFIG2)

    f = os.path.join(str(tmpdir), "default.ini")
    p = ArgumentParser(config_opts=ConfigOpts(required=True,
                                              override_arg=True,
                                              default_file=f,
                                              default_config=SAMPLE_CONFIG2))
    args = p.parse_args([])
    assert args.config is not None
    assert [i for i in args.config.items()] == [i for i in sample2.items()]
    assert args.config.get("MAIN", "mainkey") == "config2"

    args = p.parse_args(["--config-override=MAIN:mainkey=2gifnoc",
                         "--config-override=CONFIG2:key=value2",
                         "--config-override=NEWSECT:special=value",
                         ])
    assert args.config.get("MAIN", "mainkey") == "2gifnoc"
    assert args.config.get("CONFIG2", "key") == "value2"
    assert args.config.get("NEWSECT", "special") == "value"


def test_ConfigOverridesWrongParserType(tmpdir):
    from nicfit._config import addCommandLineArgs
    p = argparse.ArgumentParser()
    with pytest.raises(ValueError):
        addCommandLineArgs(p, ConfigOpts(override_arg=True))


SAMPLE_CONFIG1 = """
[MAIN]
mainkey = config1

[CONFIG1]
key = value
"""

SAMPLE_CONFIG2 = """
[MAIN]
mainkey = config2
[CONFIG2]
KEY = value
"""

SAMPLE_CONFIG3 = """
[MAIN]
mainkey = config3

[CONFIG3]
key = value
"""


def test_configoverride_argtype():
    assert _config_override("sect:key=val") == ("sect", ("key", "val"))
    for val in ("sect", "sect:key", "sect:=val", ":key=", ":="):
        with pytest.raises(ValueError):
            _config_override(val)


def test_env_config(tmpdir):
    config_path = tmpdir.join("config.ini")
    config_path.write(SAMPLE_CONFIG1)
    os.environ["CONFIG_VAR"] = str(config_path)

    c = Config("file.ini", config_env_var="CONFIG_VAR")
    assert c.sections() == ["MAIN", "CONFIG1"]
    assert c["MAIN"]["mainkey"] == "config1"
    assert c["CONFIG1"]["key"] == "value"



import pytest
import nicfit
try:
    with_curio = True
    import nicfit.curio
    from nicfit.curio import Application as CurioApp
except ImportError:
    with_curio = False
    CurioApp = object

pytestmark = pytest.mark.skipif(not with_curio, reason="curio not installed")

async def _main(args):
    args.app.retval = 2
    return 2

def atexit(app):
    app.atexit = 2

class Myapp(CurioApp):
    async def _main(self, args):
        assert args.app is self
        self.retval = 3
        return 3
    def _atexit(self):
        self.atexit = 3


def test_entryPoints():
    app = nicfit.curio.Application(_main)
    with pytest.raises(SystemExit):
        app.run([])
    assert app.retval == 2

    app = Myapp()
    with pytest.raises(SystemExit):
        app.run([])
    assert app.retval == 3

def test_atexit():
    app = nicfit.curio.Application(atexit=atexit)
    with pytest.raises(SystemExit):
        app.run()
    assert app.atexit == 2

    app = Myapp()
    with pytest.raises(SystemExit):
        app.run()
    assert app.atexit == 3



# -*- coding: utf-8 -*-
import {{ cookiecutter.py_module }}
"""
test_{{ cookiecutter.py_module }}
----------------------------------

Tests for `{{ cookiecutter.py_module }}` module.
"""


def test_metadata():
    assert {{ cookiecutter.py_module }}.version
    assert {{ cookiecutter.py_module }}.__about__.__license__
    assert {{ cookiecutter.py_module }}.__about__.__project_name__
    assert {{ cookiecutter.py_module }}.__about__.__author__
    assert {{ cookiecutter.py_module }}.__about__.__author_email__
    assert {{ cookiecutter.py_module }}.__about__.__version__
    assert {{ cookiecutter.py_module }}.__about__.__version_info__
    assert {{ cookiecutter.py_module }}.__about__.__release__
    assert {{ cookiecutter.py_module }}.__about__.__version_txt__



# -*- coding: utf-8 -*-
from collections import OrderedDict


def register(CommandSubClass):
    """A class decorator for Command classes to register in the default set."""
    name = CommandSubClass.name()
    if name in Command._all_commands:
        raise ValueError("Command already exists: " + name)
    Command._all_commands[name] = CommandSubClass
    return CommandSubClass


class CommandError(Exception):
    """Base error type for nicfit.command.Command errors."""
    def __init__(self, msg, exit_status=1):
        super().__init__(msg)
        self.exit_status = exit_status


class Command(object):
    """Base class for commands."""
    _all_commands = OrderedDict()

    @classmethod
    def name(Class):
        return Class.NAME if hasattr(Class, "NAME") else Class.__name__

    @classmethod
    def help(Class):
        return Class.HELP if hasattr(Class, "HELP") else None

    @classmethod
    def desc(Class):
        return Class.DESC if hasattr(Class, "DESC") else Class.help()

    @classmethod
    def aliases(Class):
        return Class.ALIASES if hasattr(Class, "ALIASES") else []

    def __init__(self, subparsers):
        self.subparsers = subparsers
        self.parser = self.subparsers.add_parser(self.name(),
                                                 help=self.help(),
                                                 description=self.desc(),
                                                 aliases=self.aliases())
        self._initArgParser(self.parser)
        self.parser.set_defaults(command_func=self.run)

    def _initArgParser(self, parser):
        pass

    def run(self, args):
        self.args = args
        return self._run()

    def _run(self):
        raise NotImplementedError("Must implement a _run function")

    @staticmethod
    def initAll(subparsers):
        if not Command._all_commands:
            raise ValueError("No commands have been registered")

        cmds = []
        for Cmd in Command._all_commands.values():
            cmds.append(Cmd(subparsers))
        return cmds


__all__ = ["register", "CommandError", "Command"]



#!/usr/bin/env python
import sys
import json
from pathlib import Path


def write_cc_yaml(cc_json, pre_template):
    with open(str(cc_json)) as fp:
        data = json.loads(fp.read())

    with open(str(pre_template)) as fp:
        line = fp.readline()
        while line:
            if line.strip() != "@USER_CONFIG@":
                print(line.rstrip())
            else:
                print("default_context:")
                for k in sorted(data.keys()):
                    if not k.startswith("_"):
                        val = data[k]
                        if isinstance(val, list):
                            val = val[0]
                        print("{ident}{key}: \"{{{{ cookiecutter.{key} }}}}\""
                              .format(ident=" " * 4, key=k))
            line = fp.readline()


if __name__ == "__main__":
    cc_json = sys.argv[1]
    pre_template = sys.argv[2]
    write_cc_yaml(Path(cc_json), Path(pre_template))



# -*- coding: utf-8 -*-
import sys


def pout(msg, log=None):
    """Print 'msg' to stdout, and option 'log' at info level."""
    _print(msg, sys.stdout, log_func=log.info if log else None)


def perr(msg, log=None):
    """Print 'msg' to stderr, and option 'log' at info level."""
    _print(msg, sys.stderr, log_func=log.error if log else None)


def _print(msg, file, log_func=None):
    print(msg, file=file)
    if log_func:
        log_func(msg)



import sys
import logging

from ._logging import getLogger
from ._argparse import ArgumentParser
from .util import initGetText

log = getLogger(__name__)

try:
    import ipdb as _debugger
except ImportError:                                            # pragma: nocover
    import pdb as _debugger
def _pdb():  # noqa: E302
    e, m, tb = sys.exc_info()
    _debugger.post_mortem(tb)


class Application:
    UNCAUGHT_EXCEPTION_EXIT = 126
    NO_MAIN_EXIT = 127

    def __init__(self, main_func=None, *, name=None, description=None,
                 logging_args=True, config_opts=None, version=None,
                 atexit=None, pdb_opt=False, extra_arg_parser_opts=None,
                 gettext_domain=None):
        self._main_func = main_func
        self._atexit_func = atexit

        self.log = getLogger(name) if name else log
        logging.basicConfig()

        extra_arg_parser_opts = extra_arg_parser_opts or {}
        parser = ArgumentParser(add_log_args=logging_args,
                                config_opts=config_opts,
                                prog=name, description=description,
                                **extra_arg_parser_opts)
        self.name = parser.prog

        if version:
            parser.add_argument("--version", action="version", version=version)
        if pdb_opt:
            parser.add_argument("--pdb", action="store_true", dest="debug_pdb",
                                help="Drop into 'pdb' for unhandled exceptions")

        parser.set_defaults(app=self)
        self.arg_parser = self._addArguments(parser) or parser

        if gettext_domain:
            translation = initGetText(gettext_domain, install=True)
            if translation is None:
                raise ValueError("Unable to find gettext message catalog for "
                                 "gettext_domain='{}'".format(gettext_domain))
            self.translation = translation

    def _main(self, args):
        """
        Subclasses should implement, or pass main_func to the constructor.
        """
        self.log.debug("Application._main: {args}".format(**locals()))
        assert args is self.args
        if self._main_func:
            return self._main_func(args)
        return Application.NO_MAIN_EXIT

    def _atexit(self):
        """Invoked in the 'finally' block of Application.run."""
        self.log.debug("Application._atexit")
        if self._atexit_func:
            self._atexit_func(self)

    def _addArguments(self, parser):
        return parser

    def main(self, args_list=None):
        self.log.debug("Application.main: {args_list}".format(**locals()))

        self.args = self.arg_parser.parse_args(args=args_list)
        retval = self._main(self.args)
        return retval or 0

    def run(self, args_list=None):
        """Run Application.main and exits with the return value."""
        self.log.debug("Application.run: {args_list}".format(**locals()))
        retval = None
        try:
            retval = self._run(args_list=args_list)
        except KeyboardInterrupt:
            self.log.verbose("Interrupted")                    # pragma: nocover
        except SystemExit as exit:
            self.log.verbose("Exited")
            retval = exit.code
        except Exception as ex:
            if "debug_pdb" in self.args and self.args.debug_pdb:
                _pdb()
            retval = Application.UNCAUGHT_EXCEPTION_EXIT
            self.log.exception("Uncaught exception")
            raise
        finally:
            try:
                self._atexit()
            finally:
                sys.exit(retval)

    def _run(self, args_list=None):
        self.log.debug("Application._run: {args_list}".format(**locals()))
        return self.main(args_list=args_list)

    def enableCommands(self, title="Commands", description=None,
                       add_help_subcmd=True, dest="command"):
        from .command import Command
        subs = self.arg_parser.add_subparsers(title=title,
                                              description=description,
                                              add_help_subcmd=add_help_subcmd,
                                              dest=dest)
        Command.initAll(subs)
        return subs


class AsyncApplication(Application):
    async def _main(self, args):
        self.log.debug("AsyncApplication._main: {args}".format(**locals()))
        if self._main_func:
            return await self._main_func(args)
        return self.NO_MAIN_EXIT

    async def main(self, args_list=None):
        self.log.debug("AsyncApplication: {args_list}".format(**locals()))
        self.args = self.arg_parser.parse_args(args=args_list)
        retval = await self._main(self.args)
        return retval or 0

    def _run(self, args_list=None):
        raise NotImplementedError("Implemement for a specific async API")



# -*- coding: utf-8 -*-
from collections import namedtuple


def __parse_version(v):                                       # pragma: nocover
    ver, rel = v, "final"
    for c in ("a", "b", "c"):
        parsed = v.split(c)
        if len(parsed) == 2:
            ver, rel = (parsed[0], c + parsed[1])

    v = tuple((int(v) for v in ver.split(".")))
    ver_info = namedtuple("Version", "major, minor, maint, release")(
        *(v + (tuple((0,)) * (3 - len(v))) + tuple((rel,))))
    return ver, rel, ver_info


__version__ = "{{ cookiecutter.version }}"
__release_name__ = ""
__years__ = "{{ cookiecutter.year }}"

_, __release__, __version_info__ = __parse_version(__version__)
__project_name__ = "{{ cookiecutter.project_name }}"
__project_slug__ = "{{ cookiecutter.project_slug }}"
__pypi_name__ = "{{ cookiecutter.pypi_repo_name }}"
__author__ = "{{ cookiecutter.full_name }}"
__author_email__ = "{{ cookiecutter.email }}"
__url__ = "{{ cookiecutter.web }}"
__description__ = "{{ cookiecutter.project_short_description }}"
__long_description__ = "{{ cookiecutter.project_long_description }}"
__license__ = "{{ cookiecutter.license }}"
__github_url__ = "https://github.com/{{ cookiecutter.github_username }}/{{ cookiecutter.project_slug }}",
__version_txt__ = """
%(__name__)s %(__version__)s (C) Copyright %(__years__)s %(__author__)s
This program comes with ABSOLUTELY NO WARRANTY! See LICENSE for details.
Run with --help/-h for usage information or read the docs at
%(__url__)s
""" % (locals())



# -*- coding: utf-8 -*-
import os
import sys
import gettext
import contextlib
from pathlib import Path

from ._logging import getLogger

log = getLogger(__name__)


@contextlib.contextmanager
def cd(path):
    old_path = os.getcwd()
    os.chdir(path)
    try:
        yield
    finally:
        os.chdir(old_path)


def copytree(src, dst, symlinks=True):
    """
    Modified from shutil.copytree docs code sample, merges files rather than
    requiring dst to not exist.
    """
    from shutil import copy2, Error, copystat

    names = os.listdir(src)

    if not Path(dst).exists():
        os.makedirs(dst)

    errors = []
    for name in names:
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        try:
            if symlinks and os.path.islink(srcname):
                linkto = os.readlink(srcname)
                os.symlink(linkto, dstname)
            elif os.path.isdir(srcname):
                copytree(srcname, dstname, symlinks)
            else:
                copy2(srcname, dstname)
            # XXX What about devices, sockets etc.?
        except OSError as why:
            errors.append((srcname, dstname, str(why)))
        # catch the Error from the recursive copytree so that we can
        # continue with other files
        except Error as err:
            errors.extend(err.args[0])
    try:
        copystat(src, dst)
    except OSError as why:
        # can't copy file access times on Windows
        if why.winerror is None:
            errors.extend((src, dst, str(why)))
    if errors:
        raise Error(errors)


def initGetText(domain, install=False, fallback=True):
    locale_paths = [
        Path(__file__).parent / ".." / "locale",
        Path(sys.prefix) / "share" / "locale",
    ]

    locale_dir, translation = None, None
    for locale_dir in [d for d in locale_paths if d.exists()]:
        if gettext.find(domain, str(locale_dir)):
            log.debug("Loading message catalogs from {}".format(locale_dir))
            translation = gettext.translation(domain, str(locale_dir))
            break

    if translation is None:
        # This with either throw FileNotFoundError (fallback=False) or set a
        # gettext.NullTranslations
        translation = gettext.translation(domain, str(locale_dir),
                                          fallback=fallback)
    assert translation

    if install:
        gettext.install(domain, str(locale_dir), names=["ngettext"])

    return translation



# -*- coding: utf-8 -*-
import os
import shutil
import subprocess
import collections
from uuid import uuid4
from hashlib import md5
from pathlib import Path
from tempfile import NamedTemporaryFile

import nicfit
from . import version
from .util import copytree
from .console import ansi, perr, pout
from .console.ansi import Fg, Style

try:
    from cookiecutter.main import cookiecutter
    from cookiecutter.exceptions import CookiecutterException
    import click.exceptions
except ImportError:  # pragma: nocover
    cookiecutter = None

HASH_FILE = Path("./.cookiecutter.md5")

MERGE_TOOLS = collections.OrderedDict()
MERGE_TOOLS["meld"] = None
MERGE_TOOLS["gvimdiff"] = "-geometry 169x60 -f"
MERGE_TOOLS["vimdiff"] = None


@nicfit.command.register
class Requirements(nicfit.Command):
    """
    TODO
        - infile arg

    """
    NAME = "requirements"
    ALIASES = ["reqs"]

    def _splitPkg(self, line):
        pkg, ver = line, None
        for c in ("=", ">", "<"):
            i = line.find(c)
            if i != -1:
                pkg = line[0:i]
                ver = line[i:]
                break
        return pkg, ver

    def _readReq(self, file):
        reqs = {}
        file.seek(0)
        for line in [l.strip() for l in file.readlines()
                                  if l.strip() and not l.startswith("#")]:
            pkg, version = self._splitPkg(line)
            reqs[pkg] = version
        return reqs

    def _makeReqsFile(self, filepath, reqs):
        with filepath.open("r+" if filepath.exists() else "w") as fp:
            new = {}
            curr = self._readReq(fp) if filepath.exists() else {}

            for r in [r.strip() for r in reqs if r and r.strip()]:
                pkg, ver = self._splitPkg(r)
                if ver is None and pkg in curr:
                    ver = curr[pkg]
                new[pkg] = ver

            fp.seek(0)
            fp.truncate(0)
            for pkg in sorted(new.keys()):
                ver = new[pkg] or ""
                fp.write("{pkg}{ver}\n".format(**locals()))
            pout("Wrote {}".format(filepath))

    def _run(self):
        import yaml

        reqs_file = Path("./requirements/requirements.yml")
        if not reqs_file.exists():
            pout("Nothing to do...")
            return
        reqs_dir = reqs_file.parent

        reqs_yaml = yaml.load(reqs_file.open())

        # Make split .txt files
        for name in reqs_yaml.keys():
            if reqs_yaml[name]:
                self._makeReqsFile(reqs_dir / (name + ".txt"), reqs_yaml[name])

        # Make top-level requirements.txt files
        pkg_reqs = []
        for name, pkgs in reqs_yaml.items():
            if name == "main" or name.startswith("extra_"):
                pkg_reqs += pkgs or []
        self._makeReqsFile(Path("requirements.txt"), pkg_reqs)


@nicfit.command.register
class CookieCutter(nicfit.Command):
    NAME = "cookiecutter"
    HELP = "Create a nicfit.py Python project skeleton."
    OLD_CC_USER_CONFIG = ".cookiecutter.json"
    CC_USER_CONFIG = ".cookiecutter.yml"
    ALIASES = ["cc"]

    def _initArgParser(self, parser):
        parser.add_argument("outdir", metavar="PATH",
            help="Where to output the generated project dir into")
        parser.add_argument("--config-file", metavar="PATH",
                            help="User configuration file", default=None)
        parser.add_argument("--no-input", action="store_true",
                            help="Do not prompt for parameters and only use "
                                 "{} file content".format(self.CC_USER_CONFIG))
        parser.add_argument("--no-config", action="store_true",
                            help="Use no user config (overrides --config_file)")
        parser.add_argument("--no-clone", action="store_true",
                            help="Do not clone a local repo if one is found.")
        parser.add_argument("--merge", action="store_true",
                            help="Merge CookieCutter output against local "
                            "repository (if found). Ignored when used "
                            "with --no-clone")
        parser.add_argument("--ignore-md5s", action="store_true",
                            help="Causes all files to be merged even if the "
                            "saved md5sum matches from a previous merge.")
        parser.add_argument("--extra-merge", action="append", nargs=2,
                            metavar="FILE", default=[],
                            help="Merge two files there are outside the context"
                            " of the git repo (e.g. untracked files, "
                            ".git/hooks, etc.). This option may be specified "
                            "multiple times.")
        parser.add_argument("--merge-cmd", metavar="CMD",
                            help="Merge command. Called with with 2 args: "
                                 "<src> <dest>")

    def _findTemplateDir(self):
        template_d = None
        for p in [Path(__file__).parent / "cookiecutter",
                  Path(__file__).parent.parent / "cookiecutter",
                 ]:
            if p.exists():
                template_d = p
                break
        assert template_d
        return template_d

    def _cookiecutter(self, template_d):
        try:
            cc_dir = cookiecutter(str(template_d),
                                  config_file=self.args.config_file,
                                  no_input=self.args.no_input,
                                  overwrite_if_exists=True,
                                  output_dir=self.args.outdir)
            return cc_dir
        except click.exceptions.Abort as ex:
            raise KeyboardInterrupt()  # pragma: nocover
        except CookiecutterException as ex:
            raise nicfit.CommandError("CookieCutter error: {}"
                                      .format(str(ex) if str(ex)
                                                      else ex.__class__))

    def _run(self):
        if not cookiecutter:
            raise nicfit.CommandError("CookierCutter not installed")

        cwd = Path(os.getcwd())
        clone_d = None
        if (cwd / ".git").is_dir() and not self.args.no_clone:
            pout("Cloning local repo for CookieCutter merging "
                 "(use --no-clone to disable)")
            clone_d = self._gitCloneRepo(cwd)

        # Set up user config
        if self.args.no_config:
            self.args.config_file = None
        elif not self.args.config_file:
            local_config = Path(cwd) / self.CC_USER_CONFIG
            old_local_config = Path(cwd) / self.OLD_CC_USER_CONFIG
            if local_config.is_file():
                self.args.config_file = str(local_config)
            elif old_local_config.is_file():
                self.args.config_file = str(old_local_config)
        if self.args.config_file:
            pout("Using user config ./{}, use --no-config to ignore."
                 .format(self.CC_USER_CONFIG))

        cc_dir = self._cookiecutter(self._findTemplateDir())
        if clone_d:
            copytree(str(cc_dir), str(clone_d))
            shutil.rmtree(str(cc_dir))
            os.rename(str(clone_d), str(cc_dir))

            if self.args.merge:
                self._merge(cc_dir)

    def _gitCloneRepo(self, repo_path):
        try:
            p = subprocess.run("git rev-parse --abbrev-ref HEAD", shell=True,
                               stdout=subprocess.PIPE, check=True)
            branch = str(p.stdout, "utf-8").strip()
            clone_d = Path(self.args.outdir) / str(uuid4())
            p = subprocess.run(
                "git clone --depth=1 --branch {branch} file://`pwd` '{clone_d}'"
                .format(**locals()),
                shell=True, stdout=subprocess.PIPE, check=True)
            return clone_d
        except subprocess.CalledProcessError as err:
            raise nicfit.CommandError(str(err))

    def _merge(self, cc_dir):
        md5_hashes = {}
        if HASH_FILE.exists():
            for line in [l.strip() for l in HASH_FILE.read_text().split("\n")]:
                if line:
                    values = line.rsplit(":", maxsplit=1)
                    if len(values) == 2 and values[0] and values[1]:
                        md5_hashes[values[0]] = values[1]

        try:
            p = subprocess.run("git -C \"{cc_dir}\" status --porcelain -uall"
                               .format(**locals()),
                               shell=True, stdout=subprocess.PIPE,
                               check=True)
            merge_files = []
            status = str(p.stdout, "utf-8").strip()
            for line in [s.strip() for s in status.split("\n")]:
                if line:
                    merge_files.append(tuple(line.split()))
        except subprocess.CalledProcessError as err:
            raise nicfit.CommandError(str(err))

        for st, file in merge_files + self.args.extra_merge:
            dst = Path(file)
            src = cc_dir / dst

            hasher = md5()
            try:
                hasher.update(src.read_bytes())
            except FileNotFoundError as notfound:
                perr(notfound)
                continue
            md5sum = hasher.hexdigest()
            merge_file = (self.args.ignore_md5s or
                          file not in md5_hashes or
                          md5sum != md5_hashes[file])
            pout("Comparing {} hash({}): {}"
                 .format(file, md5sum, Fg.blue("new")
                                        if merge_file else Fg.green("merged")))
            md5_hashes[file] = md5sum

            if merge_file:
                tmp_dst = None
                if not dst.exists():
                    tmp_dst = NamedTemporaryFile("w", suffix=dst.suffix,
                                                 delete=False)
                    # Write the file to exist on disk for diff and merge
                    tmp_dst.close()
                    tmp_dst = Path(tmp_dst.name)

                dst_file = str(dst if tmp_dst is None else tmp_dst)
                diffs = subprocess.run("diff '{src}' '{dst_file}' >/dev/null"
                                       .format(**locals()), shell=True)\
                                  .returncode != 0
                pout("Differences: {}".format(diffs))
                if diffs:
                    merge_cmd = self.args.merge_cmd
                    if merge_cmd is None:
                        for cmd, opts in MERGE_TOOLS.items():
                            if shutil.which(cmd):
                                merge_cmd = " ".join([cmd, opts or ""])
                                break
                    if merge_cmd is not None:
                        subprocess.run("{merge_cmd} '{src}' '{dst_file}'"
                                       .format(**locals()),
                                       shell=True, check=True)
                    else:
                        perr("Merge disabled, no merge command found. Install "
                             "a merge tool such as: {tools}.\nOr use "
                             "--merge-cmd to specify your own."
                             .format(tools=", ".join(MERGE_TOOLS.keys())))

                if tmp_dst and tmp_dst.stat().st_size == 0:
                    tmp_dst.unlink()
                elif tmp_dst:
                    # Move tmp file into place and create parent dirs
                    if not dst.parent.exists():
                        dst.parent.mkdir(0o755, parents=True)
                    shutil.move(str(tmp_dst), str(dst))

        with HASH_FILE.open("w") as hash_file:
            for f in sorted(md5_hashes.keys()):
                hash_file.write("{}:{}\n".format(f, md5_hashes[f]))


class Nicfit(nicfit.Application):
    def __init__(self):
        super().__init__(version=version, gettext_domain="nicfit.py",
                         pdb_opt=True)
        subs = self.arg_parser.add_subparsers(title="Commands",
                                              add_help_subcmd=True,
                                              dest="command")
        nicfit.Command.initAll(subs)

    def _main(self, args):
        ansi.init()
        if not args.command:
            pout(Fg.red("\m/ {} \m/"
                       .format(Style.inverse(_("Welcome")))))
            return 0

        try:
            return args.command_func(args)
        except nicfit.CommandError as err:
            perr(err)
            return err.exit_status


app = Nicfit()
if __name__ == "__main__":
    app.run()  # pragma: nocover



# -*- coding: utf-8 -*-
from . import command
from .app import Application
from ._logging import getLogger
from ._argparse import ArgumentParser
from ._config import Config, ConfigOpts
from .command import Command, CommandError
from .__about__ import __version__ as version

log = getLogger(__package__)


__all__ = ["log", "getLogger", "version",
           "Application", "ArgumentParser", "Config", "ConfigOpts",
           "command", "Command", "CommandError"]



# -*- coding: utf-8 -*-
import sys
import logging
import logging.config
import argparse
from io import StringIO

"""
TODO
"""

LOG_FORMAT = "[%(asctime)s] %(name)-25s [%(levelname)-8s]: %(message)s"
METRICS_FORMAT = "<metrics time='%(asctime)s'>%(message)s</metrics>"

logging.VERBOSE = logging.DEBUG + ((logging.INFO - logging.DEBUG) // 2)
logging.addLevelName(logging.VERBOSE, "VERBOSE")
LEVELS = [logging.DEBUG, logging.VERBOSE, logging.INFO,
          logging.WARNING, logging.ERROR, logging.CRITICAL]
LEVEL_NAMES = [logging.getLevelName(l).lower() for l in LEVELS]


def getLogger(name=None):
    OrigLoggerClass = logging.getLoggerClass()
    try:
        logging.setLoggerClass(Logger)
        return logging.getLogger(name)
    finally:
        logging.setLoggerClass(OrigLoggerClass)


class Logger(logging.getLoggerClass()):
    '''Base class for all package loggers'''

    def __init__(self, name):
        super().__init__(name)

        # Using propogation of child to parent, by default
        self.propagate = True
        self.setLevel(logging.NOTSET)

    def verbose(self, msg, *args, **kwargs):
        '''Log \a msg at 'verbose' level, debug < verbose < info'''
        self.log(logging.VERBOSE, msg, *args, **kwargs)


def _optSplit(opt):
    if ':' in opt:
        first, second = opt.split(":", maxsplit=1)
    else:
        first, second = None, opt
    return first or None, second or None


def addCommandLineArgs(arg_parser):
    """Add logging option to an ArgumentParser."""
    arg_parser.register("action", "log_levels", LogLevelAction)
    arg_parser.register("action", "log_files", LogFileAction)
    arg_parser.register("action", "log_help", LogHelpAction)

    group = arg_parser.add_argument_group("Logging options")
    group.add_argument(
        "-l", "--log-level", dest="log_levels",
        action="log_levels", metavar="LOGGER:LEVEL", default=[],
        help="Set log levels for individual loggers. See --help-logging for "
             "complete details.")

    group.add_argument(
        "-L", "--log-file", dest="log_files",
        action="log_files", metavar="LOGGER:FILE", default=[],
        help="Set log the output file for individual loggers. "
             " See --help-logging for complete details.")

    group.add_argument("--help-logging", action="log_help",
                       help=argparse.SUPPRESS)


def applyLoggingOpts(log_levels, log_files):
    """Apply logging options produced by LogLevelAction and LogFileAction.

    More often then not this function is not needed, the actions have already
    been taken during the parse, but it can be used in the case they need to be
    applied again (e.g. when command line opts take precedence but were
    overridded by a fileConfig, etc.).
    """
    for l, lvl in log_levels:
        l.setLevel(lvl)
    for l, hdl in log_files:
        for h in l.handlers:
            l.removeHandler(h)
        l.addHandler(hdl)


class LogLevelAction(argparse._AppendAction):
    """An 'action' value for log level setting options."""
    def __call__(self, parser, namespace, values, option_string=None):
        log_name, log_level = _optSplit(values)

        if log_level.lower() not in LEVEL_NAMES:
            raise ValueError("Unknown log level: {}".format(log_level))

        logger, level = (logging.getLogger(log_name),
                         getattr(logging, log_level.upper()))
        logger.setLevel(level)

        values = tuple([logger, level])
        super().__call__(parser, namespace, values, option_string=option_string)


class LogFileAction(argparse._AppendAction):
    """An 'action' value for log file setting options."""
    def __call__(self, parser, namespace, values, option_string=None):
        log_name, logpath = _optSplit(values)

        logger, logpath = logging.getLogger(log_name), logpath

        formatter = None
        handlers_logger = None
        if logger.hasHandlers():
            # Find the logger with the actual handlers attached
            handlers_logger = logger if logger.handlers else logger.parent
            while not handlers_logger.handlers:
                handlers_logger = handlers_logger.parent

            assert(handlers_logger)
            for h in list(handlers_logger.handlers):
                formatter = h.formatter
                handlers_logger.removeHandler(h)
        else:
            handlers_logger = logger

        if logpath in ("stdout", "stderr"):
            h = logging.StreamHandler(stream=sys.stdout if "out" in logpath
                                                        else sys.stderr)
        elif logpath == "null":
            h = logging.NullHandler()
        else:
            h = logging.FileHandler(logpath)

        h.setFormatter(formatter)
        handlers_logger.addHandler(h)
        handlers_logger.propagate = False

        values = tuple([logger, h])
        super().__call__(parser, namespace, values, option_string=option_string)


class LogHelpAction(argparse._HelpAction):
    def __call__(self, parser, namespace, values, option_string=None):
        print(_LOGGING_OPTS_HELP)
        parser.exit()


# FIXME: metrics does not really belong in generic version
def LOGGING_CONFIG(pkg_logger, root_level="WARN", log_format=LOG_FORMAT,
                   pkg_level="NOTSET", metrics_format=METRICS_FORMAT,
                   init_logging=False):
    cfg = """
###
#logging configuration
#https://docs.python.org/3/library/logging.config.html#configuration-file-format
###

[loggers]
keys = root, {pkg_logger}, {pkg_logger}.metrics

[handlers]
keys = console, metrics

[formatters]
keys = generic, metrics

[logger_root]
level = {root_level}
handlers = console

[logger_{pkg_logger}]
level = {pkg_level}
qualname = {pkg_logger}
; When adding more specific handlers than what exists on the root you'll
; likely want to set propagate to false.
handlers =
propagate = 1

[handler_console]
class = StreamHandler
args = (sys.stderr,)
level = NOTSET
formatter = generic

[formatter_generic]
format = {log_format}

[logger_{pkg_logger}.metrics]
level = NOTSET
qualname = {pkg_logger}.metrics
handlers = metrics
propagate = 0

[handler_metrics]
class = FileHandler
args = ("{pkg_logger}-metrics.log", "w", None, True)
level = NOTSET
formatter = metrics

[formatter_metrics]
format = {metrics_format}

""".format(**locals())
    if init_logging:
        logging.config.fileConfig(StringIO(cfg))
    return cfg


_LOGGING_OPTS_HELP = """
The command line options ``-l (--log-level)`` and ``-L (--log-file)`` can be
used to set levels and output streams for any and all loggers, therefore each
may be specified multiple times on a command line.

Each argument requires a value of the form ``VALUE`` or ``LOGGER:VALUE``.
When a LOGGER is not specified the VALUE is applied to the root logger.

Valid level names (-l and --log-level) are:
{level_names}

Note, nicfit.py loggers add a VERBOSE level, where DEBUG < VERBOSE < INFO.

Valid log file values (-L and --log-file) are any file path with the required
permissions to open and write to the file. The special values 'stdout',
'stderr', and 'null' result on logging to the console (stdout or stderr),
or /dev/null in the last case.

For example:

example.py -l info -l mylib:debug -l mylib.database:critical -L ./info.log -L mylib:./debug.log -L mylib.database:/dev/stderr

""".format(level_names=", ".join(LEVEL_NAMES))  # noqa



import asyncio
import pytest
from nicfit import aio


async def _main(args):
    args.app.retval = 2
    return 2

def atexit(app):
    app.atexit = 2


class Myapp(aio.Application):
    async def _main(self, args):
        assert args.app is self
        self.retval = 3
        return 3

    def _atexit(self):
        self.atexit = 3


def test_entryPoints():
    app = aio.Application(_main)
    with pytest.raises(SystemExit):
        app.run([])
    assert app.retval == 2

    app = Myapp()
    with pytest.raises(SystemExit):
        app.run([])
    assert app.retval == 3


def test_atexit(event_loop):
    app = aio.Application(atexit=atexit, event_loop=event_loop)
    with pytest.raises(SystemExit):
        app.run([])
    assert app.atexit == 2

    app = Myapp(event_loop=event_loop)
    with pytest.raises(SystemExit):
        app.run([])
    assert app.atexit == 3


def test_stop(event_loop):
    async def main(args):
        try:
            await asyncio.sleep(5, loop=event_loop)
        except asyncio.CancelledError:
            args.app.retval = 14
            raise
    app = aio.Application(main, event_loop=event_loop)

    def mgr():
        app.stop()
    event_loop.call_later(2, mgr)
    with pytest.raises(SystemExit):
        app.run([])
    assert app.retval == 14



# -*- coding: utf-8 -*-
import os
import sys
import pytest
from unittest.mock import patch, Mock
from nicfit.__main__ import app
from nicfit.console.ansi import Fg, Style


def test_NicfitApp_default(capfd):
    with pytest.raises(SystemExit):
        app.run([])
    out, _ = capfd.readouterr()
    assert out == Fg.red("\m/ {} \m/".format(Style.inverse("Welcome"))) + "\n"


def test_NicfitApp_invalid():
    with pytest.raises(SystemExit) as ex:
        app.run(["Side By Side"])
    assert ex.value.code == 2


def test_NicfitApp_cookiecutter_None(tmpdir):
    with patch("nicfit.__main__.cookiecutter", None):
        with pytest.raises(SystemExit) as sysexit:
            app.run(["cookiecutter", str(tmpdir)])
        assert sysexit.value.code == 1


def test_NicfitApp_cookiecutter_mock(tmpdir):
    mock_cc = Mock()
    with patch("nicfit.__main__.cookiecutter", mock_cc):
        with pytest.raises(SystemExit) as sysexit:
            app.run(["cookiecutter", "--no-clone", str(tmpdir)])

        assert sysexit.value.code == 0
        if sys.version_info[:2] >= (3, 6):
            mock_cc.assert_called_once()
        else:
            assert mock_cc.call_count == 1
        # XXX: is there "any matcher" in pytest?
        args, kwargs = mock_cc.call_args
        assert kwargs["output_dir"] == str(tmpdir)


def test_NicfitApp_cookiecutter_exception(tmpdir):
    from cookiecutter.exceptions import CookiecutterException
    mock_cc = Mock(side_effect=CookiecutterException)
    with patch("nicfit.__main__.cookiecutter", mock_cc):
        with pytest.raises(SystemExit) as sysexit:
            app.run(["cookiecutter", str(tmpdir)])
        assert sysexit.value.code == 1


@pytest.mark.skipif("TRAVIS" in os.environ,
                    reason="Failing on Travis-CI only, unshallowed_repo not "
                           "working.")
def test_NicfitApp_cookiecutter_real(tmpdir, unshallowed_repo):
    with pytest.raises(SystemExit) as sysexit:
        app.run(["cookiecutter", str(tmpdir), "--no-input"])
    assert sysexit.value.code == 0
    # XXX: validate the results?



# -*- coding: utf-8 -*-
import os
import re
from paver.easy import *
from paver.path import path


def _setup(args, capture=False):
    return sh("python setup.py %s 2> /dev/null" % args, capture=capture)


NAME, VERSION, AUTHOR, *_ = _setup("--name --version --author",
                                   capture=True).split('\n')
FULL_NAME = '-'.join([NAME, VERSION])
TESTS_DIR = "{{ cookiecutter.src_dir }}/tests"
SRC_DIRS = ["{{ cookiecutter.src_dir }}/{{ cookiecutter.py_module }}",
            TESTS_DIR]

options(
    test=Bunch(
        pdb=False,
        coverage=False,
    ),

    cookiecutter=Bunch(
        template="https://bitbucket.org/nicfit/cookiecutter-pypackage",
    ),
)

## -- misc tasks -- ##

@task
def lint():
    sh('flake8 {}'.format(" ".join(SRC_DIRS)))


## -- test tasks -- ##


@task
@cmdopts([("pdb", "",
           u"Run with all output and launch pdb on errors and failures"),
          ("coverage", "", u"Run tests with coverage analysis"),
         ])
def test(options):
    if options.test and options.test.pdb:
        debug_opts = "--pdb --pdb-failures -s"
    else:
        debug_opts = ""

    coverage_build_d = "build/tests/coverage"
    if options.test and options.test.coverage:
        coverage_opts = (
            "--cover-erase --with-coverage --cover-tests --cover-inclusive "
            "--cover-package={{ cookiecutter.project_slug }} --cover-branches --cover-html "
            "--cover-html-dir=%s %s" % (coverage_build_d, TESTS_DIR))
    else:
        coverage_opts = ""

    sh("nosetests --verbosity=1 --detailed-errors "
       "%(debug_opts)s %(coverage_opts)s" %
       {"debug_opts": debug_opts, "coverage_opts": coverage_opts})

    if coverage_opts:
        report = "%s/%s/index.html" % (os.getcwd(), coverage_build_d)
        print("Coverage Report: file://%s" % report)
        _browser(report)


@task
def test_all():
    '''Run tests for all Python versions.'''
    sh("tox")


## -- clean tasks -- ##

@task
def clean_build():
    sh("rm -fr build/")
    sh("rm -fr dist/")
    sh("rm -fr .eggs/")
    sh("find . -name '*.egg-info' -exec rm -fr {} +")
    sh("find . -name '*.egg' -exec rm -f {} +")


@task
def clean_pyc():
    sh("find . -name '*.pyc' -exec rm -f {} +")
    sh("find . -name '*.pyo' -exec rm -f {} +")
    sh("find . -name '*~' -exec rm -f {} +")
    sh("find . -name '__pycache__' -exec rm -fr {} +")


@task
def clean_test():
    sh("rm -fr .tox/")
    sh("rm -f .coverage")
    sh("rm -fr htmlcov/")


@task
def clean_patch():
    sh("find . -name '*.rej' -exec rm -f '{}' \;")
    sh("find . -name '*.orig' -exec rm -f '{}' \;")


@task
@needs("clean_build", "clean_pyc", "clean_test", "clean_patch")
def clean():
    sh("rm -fr htmlcov/")
    sh("rm -rf tags")


## -- dist tasks -- ##


@task
@needs("clean")
def dist():
    _setup("sdist")
    _setup("bdist_wheel")
    sh("ls -l dist")


# TODO
# docs
# servedocs
# release
# install
# lint
# help



## -- utils -- ##


def  _browser(file_path):
    import webbrowser
    try:
            from urllib import pathname2url
    except:
            from urllib.request import pathname2url

    webbrowser.open("file://" + pathname2url(os.path.abspath(file_path)))


## -- cookiecutter -- ##
@task
@cmdopts([
    ("template=", "",
        u" Repo URL or file path containing Cookiecutter template"),
])
def cookiecutter(options):
    from cookiecutter.main import cookiecutter
    cookiecutter(options.cookiecutter.template,
                 extra_context={{ cookiecutter|pprint }},
                 output_dir="..", no_input=True, overwrite_if_exists=True)





import sys
import pytest
from unittest.mock import patch, Mock
import nicfit.app
from nicfit.app import Application, AsyncApplication


def _main(args):
    args.app.retval = 2
    return 2


def atexit(app):
    app.atexit = 2


class Myapp(Application):
    def _main(self, args):
        assert args.app is self
        self.retval = 3
        return 3

    def _atexit(self):
        self.atexit = 3


def test_entryPoints():
    app = Application(_main)
    with pytest.raises(SystemExit):
        app.run([])
    assert app.retval == 2

    app = Myapp()
    with pytest.raises(SystemExit):
        app.run([])
    assert app.retval == 3


def test_atexit():
    app = Application(atexit=atexit)
    with pytest.raises(SystemExit):
        app.run()
    assert app.atexit == 2

    app = Myapp()
    with pytest.raises(SystemExit):
        app.run()
    assert app.atexit == 3


def test_nomain():
    app = Application()
    assert app.main([]) == Application.NO_MAIN_EXIT


def test_unhandled():
    def m(args):
        args.app.retval = Application.UNCAUGHT_EXCEPTION_EXIT
        raise ValueError("1969")
    app = Application(m)
    with pytest.raises(SystemExit):
        app.run([])
    assert app.retval == Application.UNCAUGHT_EXCEPTION_EXIT


def test_version(capfd):
    app = Application(version="6.6.6")
    with pytest.raises(SystemExit):
        app.run(["--version"])
    out, _ = capfd.readouterr()
    assert out.strip() == "6.6.6"


def test_pdb():
    app = Application(pdb_opt=True)
    app._main = Mock(side_effect=ValueError)

    with patch.object(nicfit.app._debugger, "post_mortem") as mock_pm:
        try:
            app.run(["--pdb"])
        except SystemExit:
            if sys.version_info[:2] >= (3, 6):
                mock_pm.assert_called()
            else:
                assert mock_pm.call_count != 0
        else:
            pytest.fail("Expected ValueError")

##
# AsyncApplication tests
##

def test_AsyncApp():
    app = AsyncApplication()
    with pytest.raises(NotImplementedError):
        app._run([])


async def _asyncMain(args):
    args.app.reval = 10
    return 10


@pytest.mark.asyncio
async def test_async_main():
    app = AsyncApplication()
    assert await app._main(None) == AsyncApplication.NO_MAIN_EXIT


@pytest.mark.asyncio
async def test_async_mainFunc():
    app = AsyncApplication(_asyncMain)
    assert await app.main([]) == 10



import sys
import pytest
from unittest.mock import MagicMock as Mock
from nicfit import ArgumentParser
from nicfit import ConfigOpts

LOG_LEVEL_ARGS = [("-l", "debug"), ("-lerror",),
                  ("--log-level", "debug"), ("--log-level=critical",),
                  ("-l", "log1:debug"), ("-llog2:error",),
                  ("--log-level", "log3:debug"), ("--log-level=log4:critical",),
                 ]

LOG_FILE_ARGS = [("-L", "all.log"), ("-Lall.log",),
                  ("--log-file", "all.log"), ("--log-file=all.log",),
                  ("-L", "log1:log1.log"), ("-Llog2:log2.log",),
                  ("--log-file", "log3:log3.log"), ("--log-file=log4:log4.log",),
                 ]

CONFIG_FILE_ARGS = [("-c", "config.ini"), ("-cconfig.ini",),
                    ("--config", "config.ini"), ("--config=config.ini",),
                   ]


def test_ArgumentParser_default():
    p = ArgumentParser()

    # All logging opts are off by default
    for args in LOG_LEVEL_ARGS + LOG_FILE_ARGS:
        with pytest.raises(SystemExit):
            args = p.parse_args(args)

    # All config opts are off by default
    for args in CONFIG_FILE_ARGS + [("--config-override", "SECTION:OPT=VAL")]:
        with pytest.raises(SystemExit):
            args = p.parse_args(args)


def test_default_config_arg(capfd):
    config = "[ini]\ntest = SAMPLE CONFIG"
    p = ArgumentParser(config_opts=ConfigOpts(default_config=config,
                                              default_config_opt="--DEFAULT"))
    p.exit = Mock()

    _ = p.parse_args(["--DEFAULT"])
    p.exit.assert_called_once_with(0)
    out, _ = capfd.readouterr()
    assert out == config + "\n"


def test_default_config_opt_no_arg():
    with pytest.raises(ValueError):
        ArgumentParser(config_opts=ConfigOpts(default_config=None,
                                              default_config_opt="--DEFAULT"))


def test_ArgumentParser_add_subparsers():
    with pytest.raises(SystemExit):
        ArgumentParser().parse_args(["help"])

    p = ArgumentParser()
    p.add_subparsers(add_help_subcmd=True)
    args = p.parse_args(["help"])
    assert args.command is None
    assert args.func is not None

    p = ArgumentParser()
    p.add_subparsers(add_help_subcmd=True)
    args = p.parse_args(["help", "cmd"])
    assert args.command is "cmd"
    assert args.func is not None
    # Help func may called parse_args again, or print_help. mock em.
    p.parse_args = Mock()
    p.print_help = Mock()
    with pytest.raises(SystemExit):
        args.func(args, Mock())
    p.parse_args.assert_called_with(["cmd", "--help"])
    p.print_help.assert_not_called()

    p.parse_args.reset_mock()
    p.print_help.reset_mock()
    with pytest.raises(SystemExit):
        args.command = None
        args.func(args, Mock())
    if sys.version_info[:2] >= (3, 6):
        p.print_help.assert_called_once()
    else:
        assert p.print_help.call_count == 1
    p.parse_args.assert_not_called()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys
import warnings
from setuptools import setup, find_packages
from setuptools.command.install import install

classifiers = [
    "Intended Audience :: {{ cookiecutter.intended_audience }}",
    "Operating System :: POSIX",
    "Natural Language :: English",
{%- if cookiecutter.license == "MIT" %}
    "License :: OSI Approved :: MIT License",
{%- elif cookiecutter.license == "BSD-3" %}
    "License :: OSI Approved :: BSD License",
{%- elif cookiecutter.license == "GNU GPL v3.0" %}
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
{%- elif cookiecutter.license == "GNU GPL v2.0" %}
    "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
{%- elif cookiecutter.license == "GNU LGPL v3.0" %}
    "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
{%- elif cookiecutter.license == "GNU LGPL v2.0" %}
    "License :: OSI Approved :: GNU Lesser General Public License v2 (LGPLv2)",
{%- elif cookiecutter.license == "Apache Software License 2.0" %}
    "License :: OSI Approved :: Apache Software License",
{%- elif cookiecutter.license == "Mozilla Public License 2.0" %}
    "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
{%- endif %}
    "Programming Language :: Python",
{%- if cookiecutter.py26 == "yes" %}
    "Programming Language :: Python :: 2.6",
{%- endif %}
{%- if cookiecutter.py27 == "yes" %}
    "Programming Language :: Python :: 2.7",
{%- endif %}
{%- if cookiecutter.py33 == "yes" %}
    "Programming Language :: Python :: 3.3",
{%- endif %}
{%- if cookiecutter.py34 == "yes" %}
    "Programming Language :: Python :: 3.4",
{%- endif %}
{%- if cookiecutter.py35 == "yes" %}
    "Programming Language :: Python :: 3.5",
{%- endif %}
{%- if cookiecutter.py36 == "yes" %}
    "Programming Language :: Python :: 3.6",
{%- endif %}
    # XXX Remove to enable PyPi uploads
    "Private :: Do Not Upload",
]


def getPackageInfo():
    info_dict = {}
    info_keys = ["version", "name", "author", "author_email", "url", "license",
                 "description", "release_name", "github_url"]
    key_remap = {"name": "pypi_name"}

    # __about__
    with open(os.path.join(os.path.abspath(os.path.dirname(__file__)),
                           "{{ cookiecutter.src_dir }}",
                           "{{ cookiecutter.py_module }}",
                           "__about__.py")) as infof:
        for line in infof:
            for what in info_keys:
                rex = re.compile(r"__{what}__\s*=\s*['\"](.*?)['\"]"
                                  .format(what=what if what not in key_remap
                                                    else key_remap[what]))

                m = rex.match(line.strip())
                if not m:
                    continue
                info_dict[what] = m.groups()[0]

    if sys.version_info[:2] >= (3, 4):
        vparts = info_dict["version"].split("-", maxsplit=1)
    else:
        vparts = info_dict["version"].split("-", 1)
    info_dict["release"] = vparts[1] if len(vparts) > 1 else "final"

    # Requirements
    requirements, extras = requirements_yaml()
    info_dict["install_requires"] = requirements["main"] \
                                        if "main" in requirements else []
    info_dict["tests_require"] = requirements["test"] \
                                     if "test" in requirements else []
    info_dict["extras_require"] = extras

    # Info
    readme = ""
    if os.path.exists("README.rst"):
        with open("README.rst") as readme_file:
            readme = readme_file.read()
    history = ""
    if os.path.exists("HISTORY.rst"):
        with open("HISTORY.rst") as history_file:
            history = history_file.read().replace(".. :changelog:", "")
    info_dict["long_description"] = readme + "\n\n" + history

    return info_dict, requirements


def requirements_yaml():
    EXTRA = "extra_"
    reqs = {}
    reqfile = os.path.join("requirements", "requirements.yml")
    if os.path.exists(reqfile):
        with open(reqfile) as fp:
            curr = None
            for line in [l for l in fp.readlines() if l.strip()]:
                if curr is None or line.lstrip()[0] != "-":
                    curr = line.split(":")[0]
                    reqs[curr] = []
                else:
                    line = line.strip()
                    assert line[0] == "-"
                    r = line[1:].strip()
                    if r:
                        reqs[curr].append(r)

    return (reqs, {x[len(EXTRA):]: vals
                     for x, vals in reqs.items() if x.startswith(EXTRA)})


class PipInstallCommand(install, object):
    def run(self):
        reqs = " ".join(["'%s'" % r for r in PKG_INFO["install_requires"]])
        os.system("pip install " + reqs)
        # XXX: py27 compatible
        return super(PipInstallCommand, self).run()


PKG_INFO, REQUIREMENTS = getPackageInfo()
if PKG_INFO["release"].startswith("a"):
    #classifiers.append("Development Status :: 1 - Planning")
    #classifiers.append("Development Status :: 2 - Pre-Alpha")
    classifiers.append("Development Status :: 3 - Alpha")
elif PKG_INFO["release"].startswith("b"):
    classifiers.append("Development Status :: 4 - Beta")
else:
    classifiers.append("Development Status :: 5 - Production/Stable")
    #classifiers.append("Development Status :: 6 - Mature")
    #classifiers.append("Development Status :: 7 - Inactive")

gz = "{name}-{version}.tar.gz".format(**PKG_INFO)
PKG_INFO["download_url"] = (
    "{github_url}/releases/downloads/v{version}/{gz}"
    .format(gz=gz, **PKG_INFO)
)


def package_files(directory, prefix=".."):
    paths = []
    for (path, _, filenames) in os.walk(directory):
        if "__pycache__" in path:
            continue
        for filename in filenames:
            if filename.endswith(".pyc"):
                continue
            paths.append(os.path.join(prefix, path, filename))
    return paths


if sys.argv[1:] and sys.argv[1] == "--release-name":
    print(PKG_INFO["release_name"])
    sys.exit(0)
else:
    # The extra command line options we added cause warnings, quell that.
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", message="Unknown distribution option")
        warnings.filterwarnings("ignore", message="Normalizing")
        setup(classifiers=classifiers,
              package_dir={"": "{{ cookiecutter.src_dir }}"},
              packages=find_packages("{{ cookiecutter.src_dir }}",
                                     exclude=["tests", "tests.*"]),
              zip_safe=False,
              platforms=["Any"],
              keywords=["{{ cookiecutter.project_slug }}"],
              test_suite="{{ cookiecutter.src_dir }}/tests",
              include_package_data=True,
              package_data={},
              entry_points={
                  "console_scripts": [
                      "{{ cookiecutter.py_module }} = {{ cookiecutter.py_module }}.__main__:app.run",
                  ]
              },
              cmdclass={
                  "install": PipInstallCommand,
              },
              **PKG_INFO
        )



import sys
import pytest
from unittest.mock import MagicMock as Mock
from nicfit.command import Command, CommandError, register


@pytest.fixture
def emptycommands():
    Command._all_commands.clear()


def test_initAllEmpty(emptycommands):
    with pytest.raises(ValueError):
        Command.initAll(Mock())


def test_register(emptycommands):
    @register
    class c1(Command): pass # noqa

    @register
    class c2(Command): pass # noqa

    assert len(Command._all_commands) == 2
    Command.initAll(Mock(spec=["add_parser"]))


def test_dup_register(emptycommands):
    @register
    class c1(Command): pass # noqa

    @register
    class c2(Command): pass # noqa

    with pytest.raises(ValueError):
        @register
        class c2dup(Command):
            NAME = "c2"

    assert len(Command._all_commands) == 2
    Command.initAll(Mock(spec=["add_parser"]))


def test_run_notimplemented(emptycommands):
    @register
    class TestCommand(Command):
        pass
    Command.initAll(Mock(spec=["add_parser"]))

    with pytest.raises(NotImplementedError):
        TestCommand(Mock()).run(None)


def test_run(emptycommands):
    @register
    class TestCommand(Command):
        NAME = "test"
        _initArgParser = Mock()
        _run = Mock()

    mock_args = Mock()
    mock_parser = Mock()
    mock_parser.set_defaults = Mock()
    mock_subparser = Mock()
    mock_subparser.add_parser = Mock(return_value=mock_parser)

    cmd = Command.initAll(mock_subparser)[0]
    assert len(Command._all_commands) == 1

    assert cmd.subparsers is mock_subparser
    assert cmd.parser is mock_parser
    cmd._initArgParser.assert_called_once_with(mock_parser)

    cmd.run(mock_args)
    if sys.version_info[:2] >= (3, 6):
        cmd._run.assert_called_once()
    else:
        assert cmd._run.call_count == 1
    assert cmd.args is mock_args
    mock_parser.set_defaults.assert_called_once_with(command_func=cmd.run)


def test_run_exit_status(emptycommands):
    mock_subparser = Mock(spec=["add_parser"])

    @register
    class CNone(Command):
        def _run(self):
            pass

    assert CNone(mock_subparser).run(None) is None

    @register
    class C0(Command):
        def _run(self):
            return 0

    assert C0(mock_subparser).run(None) == 0

    @register
    class C99(Command):
        def _run(self):
            return 99

    assert C99(mock_subparser).run(None) == 99

    @register
    class CErr(Command):
        def _run(self):
            raise CommandError("The Faint", 6)

    with pytest.raises(CommandError) as err:
        assert CErr(mock_subparser).run(None) == 6
    assert str(err.value) == "The Faint"
    assert err.value.exit_status == 6

    @register
    class CExc(Command):
        def _run(self):
            raise ValueError("Danse Macabre")

    with pytest.raises(ValueError):
        CExc(mock_subparser).run(None)



import sys
import pytest
import logging
import logging.config
from uuid import uuid4
from io import StringIO
from nicfit._logging import LOGGING_CONFIG, getLogger
from nicfit import _logging
from nicfit import ArgumentParser
from unittest.mock import patch, Mock, call


def test_log():
    # No handlers by default
    mylog = getLogger("test")
    assert isinstance(mylog, _logging.Logger)
    assert mylog.name == "test"
    assert len(mylog.handlers) == 0

    pkglog = getLogger("nicfit")
    assert isinstance(mylog, _logging.Logger)
    assert pkglog.name == "nicfit"

    for logger in [mylog, pkglog]:
        assert(logger.propagate)
        assert(isinstance(logger, _logging.Logger))
        with patch.object(logger, "log") as mock_log:
            logger.verbose("Honey's Dead")
            mock_log.assert_called_with(logging.VERBOSE, "Honey's Dead")


def test_logLevelArguments():
    p = ArgumentParser(add_log_args=True)

    # Invalids
    for args, Exc in [(["-l"], SystemExit), (["--log-level"], SystemExit),
                      (["-l", "Vision-InTheBlinkOfAnEye"], ValueError),
                      (["-l", "debug:TheOnlyOne"], ValueError),
                     ]:
        with pytest.raises(Exc):
            p.parse_args(args)

    # Level sets on root logger.
    for args, level in [(["-l", "debug"], logging.DEBUG),
                        (["-lerror"], logging.ERROR),
                        (["--log-level", "warning"], logging.WARN),
                        (["--log-level=critical"], logging.CRITICAL),
                        (["--log-level=verbose"], logging.VERBOSE),
                       ]:
        p.parse_args(args)
        logger = getLogger()
        assert logger.getEffectiveLevel() == level

    # Level sets on other logger.
    for args, level, log in [
        (["-l", "Venom:debug"], logging.DEBUG, "Venom"),
        (["-lWatain:eRroR"], logging.ERROR, "Watain"),
        (["--log-level", "l1:WARNING"], logging.WARN, "l1"),
        (["--log-level=nicfit:critical"], logging.CRITICAL, "nicfit"),
        (["--log-level=eyeD3:verbose"], logging.VERBOSE, "eyeD3"),
    ]:
        logger = getLogger(log)
        with patch.object(logger, "setLevel") as mock_log:
            p.parse_args(args)
        mock_log.assert_called_with(level)


def test_logFileArguments(tmpdir):
    p = ArgumentParser(add_log_args=True)

    # Invalids
    for args, Exc in [(["-L"], SystemExit), (["--log-file"], SystemExit),
                     ]:
        with pytest.raises(Exc):
            p.parse_args(args)

    # Load up a logger with a log of handlers to
    parentlog = getLogger("parentlog")
    parentlog.addHandler(Mock())
    parentlog.addHandler(Mock())
    _ = getLogger("parentlog.child1")

    for args in [["-L", "<logfile>"],
                 ["-L<logfile>"],
                 ["--log-file", "<logfile>"],
                 ["--log-file=<logfile>"],
                 ["--log-file=mishmash:<logfile>"],
                 ["--log-file", "hallow:<logfile>"],
                 ["-Lsomelog:<logfile>"],
                 ["-Lparentlog.child1.child2:<logfile>"],
                 ["-L", "LiveLikeAnAngel:<logfile>"],
                ]:
        f = tmpdir / str(uuid4())
        assert not f.exists()
        args = [a.replace("<logfile>", str(f)) for a in args]
        a = p.parse_args(args)
        assert f.exists()

    for args in [["-L", "Bliss:stderr"],
                 ["--log-file", "Outkast:stdout"],
                 ["--log-file=Gargabe:null"],
                ]:
        _ = p.parse_args(args)
        # FIXME: actually test


def test_FileConfig():
    cfg = LOGGING_CONFIG("INl3agueWitS4t4n")
    cfg_file = StringIO(cfg)
    logging.config.fileConfig(cfg_file)


def test_FileConfig_auto():
    with patch("logging.config.fileConfig") as mock_fileConfig:
        LOGGING_CONFIG("INl3agueWitS4t4n", init_logging=True)
        if sys.version_info[:2] >= (3, 6):
            mock_fileConfig.assert_called()
        else:
            assert mock_fileConfig.call_count != 0


def test_optSplit():
    assert(_logging._optSplit("foo") == (None, "foo"))
    assert(_logging._optSplit("foo:bazz") == ("foo", "bazz"))
    assert(_logging._optSplit("foo:bar:bazz") == ("foo", "bar:bazz"))
    assert(_logging._optSplit(":bazz") == (None, "bazz"))


def test_applyLogOpts():
    mock_setLevel = Mock()
    mock_addHandler = Mock()

    mock_def_logger = Mock()
    mock_def_logger.handlers = ["DummyHandler", "AnotherDummyHandler"]
    mock_eyeD3_logger = Mock()
    mock_eyeD3_logger.handlers = []
    for l in [mock_def_logger, mock_eyeD3_logger]:
        l.setLevel = mock_setLevel
        l.addHandler = mock_addHandler

    p = ArgumentParser(add_log_args=True)
    args = p.parse_args(["-ldebug", "-L", "log.log", "--log-level", "warning",
                         "--log-level=eyeD3:error"])
    for arg_list in [args.log_levels, args.log_files]:
        for i in range(len(arg_list)):
            if arg_list[i][0] is getLogger():
                mock_logger = mock_def_logger
            elif arg_list[i][0] is getLogger("eyeD3"):
                mock_logger = mock_eyeD3_logger
            else:
                pytest.fail("Unhandled logger")
            arg_list[i] = (mock_logger, arg_list[i][1])

    args.applyLoggingOpts(args.log_levels, args.log_files)
    assert mock_setLevel.call_count == 3
    mock_setLevel.assert_has_calls([call(logging.DEBUG), call(logging.WARN),
                                    call(logging.ERROR)])
    assert mock_addHandler.call_count == 1
    assert getLogger().getEffectiveLevel() == logging.WARN
    assert getLogger("eyeD3").getEffectiveLevel() == logging.ERROR



# -*- coding: utf-8 -*-
import pytest
import nicfit
"""
test_nicfit
----------------------------------

Tests for `nicfit` module.
"""


def test_metadata():
    assert nicfit.version
    assert nicfit.__about__.__license__
    assert nicfit.__about__.__project_name__
    assert nicfit.__about__.__author__
    assert nicfit.__about__.__author_email__
    assert nicfit.__about__.__version__
    assert nicfit.__about__.__version_info__
    assert nicfit.__about__.__release__
    assert nicfit.__about__.__version_txt__


def test_parse_version():
    from nicfit.__about__ import __parse_version
    assert __parse_version("0") == ("0", "final", (0, 0, 0, "final"))
    assert __parse_version("1a1") == ("1", "a1", (1, 0, 0, "a1"))
    assert __parse_version("1.2b1") == ("1.2", "b1", (1, 2, 0, "b1"))
    assert __parse_version("1.2.3c1") == ("1.2.3", "c1", (1, 2, 3, "c1"))
    assert __parse_version("2.0") == ("2.0", "final", (2, 0, 0, "final"))

    for invalid in ["3.0.3d1", "a.b.c"]:
        with pytest.raises(ValueError):
            __parse_version(invalid)



import re
import sys


MODULE_REGEX = r"^[_a-zA-Z][_a-zA-Z0-9]+$"
module_name = "{{ cookiecutter.py_module }}"

if not re.match(MODULE_REGEX, module_name):
    print("ERROR: The project slug (%s) is not a valid." % module_name)
    # Exit to cancel project
    sys.exit(1)

user_config = """
default_context:
    bitbucket_repo: "{{ cookiecutter.bitbucket_repo }}"
    bitbucket_url: "{{ cookiecutter.bitbucket_url }}"
    bitbucket_username: "{{ cookiecutter.bitbucket_username }}"
    default_locale: "{{ cookiecutter.default_locale }}"
    email: "{{ cookiecutter.email }}"
    full_name: "{{ cookiecutter.full_name }}"
    gettext_domain: "{{ cookiecutter.gettext_domain }}"
    github_repo: "{{ cookiecutter.github_repo }}"
    github_url: "{{ cookiecutter.github_url }}"
    github_username: "{{ cookiecutter.github_username }}"
    intended_audience: "{{ cookiecutter.intended_audience }}"
    license: "{{ cookiecutter.license }}"
    project_long_description: "{{ cookiecutter.project_long_description }}"
    project_name: "{{ cookiecutter.project_name }}"
    project_short_description: "{{ cookiecutter.project_short_description }}"
    project_slug: "{{ cookiecutter.project_slug }}"
    py26: "{{ cookiecutter.py26 }}"
    py27: "{{ cookiecutter.py27 }}"
    py33: "{{ cookiecutter.py33 }}"
    py34: "{{ cookiecutter.py34 }}"
    py35: "{{ cookiecutter.py35 }}"
    py36: "{{ cookiecutter.py36 }}"
    py37: "{{ cookiecutter.py37 }}"
    py_module: "{{ cookiecutter.py_module }}"
    pyapp_type: "{{ cookiecutter.pyapp_type }}"
    pypi_repo_name: "{{ cookiecutter.pypi_repo_name }}"
    pypi_username: "{{ cookiecutter.pypi_username }}"
    release_date: "{{ cookiecutter.release_date }}"
    src_dir: "{{ cookiecutter.src_dir }}"
    use_bitbucket: "{{ cookiecutter.use_bitbucket }}"
    use_github: "{{ cookiecutter.use_github }}"
    use_make: "{{ cookiecutter.use_make }}"
    use_paver: "{{ cookiecutter.use_paver }}"
    use_pypi_deployment_with_travis: "{{ cookiecutter.use_pypi_deployment_with_travis }}"
    use_pytest: "{{ cookiecutter.use_pytest }}"
    use_rtd: "{{ cookiecutter.use_rtd }}"
    use_travis_ci: "{{ cookiecutter.use_travis_ci }}"
    version: "{{ cookiecutter.version }}"
    web: "{{ cookiecutter.web }}"
    year: "{{ cookiecutter.year }}"
"""

with open(".cookiecutter.yml", "w") as fp:
    fp.write(user_config)



import os
import sys

_USE_ANSI = False


def init(enabled=os.isatty(sys.stdout.fileno())):
    global _USE_ANSI

    if enabled and (("TERM" in os.environ and os.environ["TERM"] == "dumb") or
                    ("OS" in os.environ and os.environ["OS"] == "Windows_NT")):
        raise ValueError("ANSI not suported on dumb terminals or Windows")
    else:
        _USE_ANSI = enabled


class FgPalette:
    (GREY,
     RED,
     GREEN,
     YELLOW,
     BLUE,
     MAGENTA,
     CYAN,
     WHITE) = [*range(30, 38)]
    RESET = 39


class BgPalette:
    (GREY,
     RED,
     GREEN,
     YELLOW,
     BLUE,
     MAGENTA,
     CYAN,
     WHITE) = [*range(40, 48)]
    RESET = 49


class StylePalette:
    (RESET_ALL,
     BRIGHT,
     DIM,
     ITALICS,
     UNDERLINE,
     BLINK_SLOW,
     BLINK_FAST,
     INVERSE) = [*range(0, 8)]
    STRIKE_THRU = 9

    (RESET_BRIGHT,
     RESET_ITALICS,
     RESET_UNDERLINE,
     RESET_BLINK_SLOW,
     RESET_BLINK_FAST,
     RESET_INVERSE) = [*range(22, 28)]
    RESET_STRIKE_THRU = 29

    RESET_DIM = RESET_BRIGHT


class AnsiCodes(object):
    _CSI = "\033["

    def __init__(self, codes):
        def code_to_chars(code):
            return AnsiCodes._CSI + str(code) + 'm'

        for name in dir(codes):
            if name == name.upper():
                value = getattr(codes, name)
                setattr(self, name, code_to_chars(value))

                # Add color function
                for reset_name in ("RESET_%s" % name, "RESET"):
                    if hasattr(codes, reset_name):
                        reset_value = getattr(codes, reset_name)
                        setattr(self, "%s" % name.lower(),
                                AnsiCodes._mkfunc(code_to_chars(value),
                                                  code_to_chars(reset_value)))
                        break

    @staticmethod
    def _mkfunc(color, reset):
        def _cwrap(text, *styles):
            if not _USE_ANSI:
                return text

            s = u""
            for st in styles:
                s += st
            s += color + text + reset
            if styles:
                s += Style.RESET_ALL
            return s
        return _cwrap

    def __getattribute__(self, name):
        attr = super(AnsiCodes, self).__getattribute__(name)
        if (hasattr(attr, "startswith") and
                attr.startswith(AnsiCodes._CSI) and
                not _USE_ANSI):
            return ""
        else:
            return attr

    def __getitem__(self, name):
        return getattr(self, name.upper())


Fg = AnsiCodes(FgPalette)
Bg = AnsiCodes(BgPalette)
Style = AnsiCodes(StylePalette)

"""

Examples:
---------
::

    from nicfit.console.ansi import init, Fg, Bg, Style

    ansi.init(True)

    print(Fg.RED + "\m/ \m/" + Fg.RESET)
    print(Style.BRIGHT + Fg.RED + "\m/ \m/" + Fg.RESET)
    print(Fg.blue("\m/ \m/"))
    print(Style.BLINK_SLOW + Fg.green("\m/ \m/"))
    print(Style.DIM + Fg.green("\m/ \m/"))
    print(Fg.yellow("\m/ \m/", Style.BRIGHT, Style.UNDERLINE, Style.ITALICS))
    print("{b}\{g}m{r}{b}/{r}".format(b=Fg.BLUE, g=Fg.GREEN, r=Fg.RESET))
    print(Bg.green(Fg.yellow("\{}/".format(Style.strike_thru("mmmm")))))
    print("%(BLUE)sNice%(RESET)s" % Fg)

    # TODO
    ################################################################################
"""

__all__ = ["Fg", "Bg", "Style"]



# -*- coding: utf-8 -*-
{%- if cookiecutter.pyapp_type == "asyncio" -%}
{%- set async = "async" -%}
{%- set appmod = "nicfit.aio" -%}
{%- else -%}
{%- set async = "" -%}
{%- set appmod = "nicfit" -%}
{%- endif %}
from {{ appmod }} import Application
from nicfit.console import pout
from . import version


{{ async }} def main(args):
    pout("\m/")


app = Application(main, version=version,
{%- if cookiecutter.gettext_domain != "None" %}
                  gettext_domain="{{ cookiecutter.gettext_domain }}")
{% else %}
                  gettext_domain=None)
{% endif %}
if __name__ == "__main__":
    app.run()



#!/usr/bin/env python
import os
import shutil
import datetime
from pathlib import Path

PROJECT_DIRECTORY = os.path.realpath(os.path.curdir)


def remove_file(filepath):
    os.remove(str(Path(PROJECT_DIRECTORY) / filepath))


def replace_contents(filename, what, replacement):
    with filename.open() as fh:
        changelog = fh.read()
    with filename.open('w') as fh:
        fh.write(changelog.replace(what, replacement))


def copytree(src, dst, symlinks=False, ignore=None):
    if not os.path.exists(dst):
        os.makedirs(dst)
    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            copytree(s, d, symlinks, ignore)
        else:
            if (not os.path.exists(d) or
                    os.stat(s).st_mtime - os.stat(d).st_mtime > 1):
                shutil.copy2(s, d)


if __name__ == "__main__":
    today = datetime.date.today()

    if '{{ cookiecutter.use_github }}' == 'yes':
        remove_file('.hgignore')
        if not Path(".git").exists():
            os.system("git init .")
        shutil.move("git-commit-msg", ".git/hooks/commit-msg")
    else:
        remove_file('.gitignore')
        remove_file('git-commit-msg')

    for path in [Path("HISTORY.rst"),
                 Path('{{ cookiecutter.py_module }}') / "__about__.py",
                 Path('docs') / 'conf.py',
                ]:
        replace_contents(path, '<TODAY>', today.strftime("%Y-%m-%d"))
        replace_contents(path, '<YEAR>', today.strftime("%Y"))

    tox_envlist = []
    for py, on in [("py26", "{{ cookiecutter.py26 }}"),
                   ("py27", "{{ cookiecutter.py27 }}"),
                   ("py33", "{{ cookiecutter.py33 }}"),
                   ("py34", "{{ cookiecutter.py34 }}"),
                   ("py35", "{{ cookiecutter.py35 }}"),
                   ("py36", "{{ cookiecutter.py36 }}"),
                  ]:
        if on == "yes":
            tox_envlist.append(py)
    replace_contents(Path("tox.ini"), '@TOX_ENVLIST@',
                     ", ".join(tox_envlist))

    if '{{ cookiecutter.use_travis_ci }}' == 'no':
        remove_file('.travis.yml')
    if '{{ cookiecutter.use_pypi_deployment_with_travis }}' == 'no':
        remove_file('travis_pypi_setup.py')

    if '{{ cookiecutter.use_paver }}' == 'no':
        remove_file('pavement.py')
    if '{{ cookiecutter.use_make }}' == 'no':
        remove_file('Makefile')

    for f in Path("licenses").iterdir():
        f.unlink()
    Path("licenses").rmdir()

    if "{{ cookiecutter.src_dir }}" != ".":
        src_d = Path("{{ cookiecutter.src_dir}}")
        if not src_d.exists():
            src_d.mkdir(parents=True)
        copytree("{{ cookiecutter.py_module }}",
                 str(src_d / "{{ cookiecutter.py_module }}"))
        copytree("tests", str(src_d / "tests"))
        shutil.rmtree("{{ cookiecutter.py_module }}")
        shutil.rmtree("tests")

    domain = "{{ cookiecutter.gettext_domain }}"
    if domain == "None":
        shutil.rmtree("locale")
    else:
        gk = Path("locale/.gitkeep")
        if gk.exists():
            gk.unlink()



# -*- coding: utf-8 -*-
import os.path
import argparse
import configparser
from pathlib import Path
from collections import namedtuple


class Config(configparser.ConfigParser):
    """Class for storing, reading, and writing config."""
    def __init__(self, filename, *, config_env_var=None, **kwargs):
        super().__init__(**kwargs)

        if (config_env_var and config_env_var in os.environ and
                Path(os.environ[config_env_var]).exists()):
            with open(os.environ[config_env_var]) as confp:
                self.read_file(confp)

        self.filename = Path(os.path.expandvars(filename)).expanduser() \
                            if filename else None

    def read(self, filenames=None, encoding=None, touch=False):
        super().read(filenames or [], encoding=encoding)

        if not self.filename.exists() and touch:
            self.filename.touch()

        with open(str(self.filename), encoding=encoding) as fp:
            self.read_file(fp, source=str(self.filename))

        return self

    def write(self, fileobject=None, space_around_delimiters=True):
        if fileobject is None:
            fp = open(str(self.filename), 'w')
        else:
            fp = fileobject

        try:
            super().write(fp, space_around_delimiters=space_around_delimiters)
        finally:
            if fileobject is None:
                fp.close()


class ConfigFileType(argparse.FileType):
    """ArgumentParser ``type`` for loading ``Config`` objects."""
    def __init__(self, config_opts=None, encoding="utf-8"):
        super().__init__(mode='r')
        self._encoding = encoding
        self._opts = config_opts or ConfigOpts()

    def __call__(self, filename):
        if not filename and not self._opts.default_config:
            return None

        assert(issubclass(self._opts.ConfigClass, Config))
        config = self._opts.ConfigClass(filename,
                                        config_env_var=self._opts.env_var)

        if self._opts.default_config:
            config.read_string(self._opts.default_config)

        if filename:
            try:
                fp = super().__call__(filename)
                config.readfp(fp)
            except Exception as ex:
                if not self._opts.default_config:
                    raise

        return config


class ConfigOpts(namedtuple("_ConfigOptions", ["required",
                                               "default_file",
                                               "default_config",
                                               "override_arg",
                                               "ConfigClass",
                                               "default_config_opt",
                                               "env_var",
                                              ])):
    # XXX: attrs lib could make this go away
    def __new__(cls, required=False, default_file=None, default_config=None,
                override_arg=False, ConfigClass=Config,
                default_config_opt=None, env_var=None):
        return super().__new__(cls, required, default_file, default_config,
                               override_arg, ConfigClass, default_config_opt,
                               env_var)


def addCommandLineArgs(arg_parser, opts):
    from ._argparse import ArgumentParser

    g = arg_parser.add_argument_group("Configuration options")
    if opts.required:
        arg_parser.add_argument(
            "config", default=opts.default_file,
            help="Configuration file (ini file format).",
            type=ConfigFileType(opts),
            nargs="?" if opts.default_file else None)
    else:
        g.add_argument("-c", "--config", dest="config", metavar="file.ini",
                       type=ConfigFileType(opts),
                       default=ConfigFileType(opts)(opts.default_file),
                       help="Configuration file (ini file format).")

    if opts.override_arg:
        if not isinstance(arg_parser, ArgumentParser):
            raise ValueError("nicfit.ArgumentParser type required for "
                             "--config-override support.")

        g.add_argument("--config-override", dest="config_overrides",
                       action="append", metavar="SECTION:OPTION=VALUE",
                       type=_config_override,
                       help="Overrides the value for configuration OPTION in "
                            "[SECTION].")

    if opts.default_config_opt:
        if opts.default_config is None:
            raise ValueError("ConfigOpts.default_config_opt requires a value "
                             "in ConfigOpts.default_config")
        g.add_argument(opts.default_config_opt, dest="config_show_default",
                       action="store_true",
                       help="Prints the default configuration.")


def _config_override(s):
    sect, rhs = s.split(':', 1)
    key, val = rhs.split('=', 1)
    if not sect or not key:
        raise ValueError("section and key required")
    return (sect, (key, val))
